@@ -1,11 +1,14 @@
 # 2016. 1. 10 by Hans Roh hansroh@gmail.com
 
-__version__ = "0.7.7.3"
+__version__ = "0.7.8.11"
 version_info = tuple (map (lambda x: not x.isdigit () and x or int (x),  __version__.split (".")))
-import threading
+
+import os, sys
+import asyncore
+import timeit
+import time, math, random
 from . import lifetime, queue, request_builder, response_builder, stubproxy
 from .lib import logger as logger_f
-from .lib.athreads import trigger
 from .client import socketpool
 from .dbapi import dbpool
 from .client import adns, asynconnect
@@ -17,11 +20,6 @@ from .protocols.http import request_handler, response as http_response
 from .protocols import http2
 from .protocols.http2 import H2_PROTOCOLS
 from .dbapi import request as dbo_request
-import os
-import asyncore
-import timeit
-import time, math, random
-import sys
 
 DEBUG = 0
 
@@ -87,6 +85,7 @@ def configure (
 	allow_redirects = True,
 	qrandom = False,
 	use_pool = True,
+	tracking = False,
 	backend = False,
 ):
 	global _logger, _cb_gateway, _concurrent, _initialized, _timeout
@@ -118,7 +117,8 @@ def configure (
 	if not force_http1:
 		_concurrent = workers * http2_constreams
 	elif http2_constreams:
-		_logger ("parameter http2_constreams is ignored", "warn")	
+		pass
+		#_logger ("parameter http2_constreams is ignored", "warn")	
 				
 	if callback:
 		_cb_gateway = callback
@@ -129,10 +129,12 @@ def configure (
 	client.set_timeout (timeout)
 	dbapi.set_timeout (timeout)
 	
-	socketpool.create (_logger, backend = backend)
+	socketpool.create (_logger, backend = backend, use_pool = use_pool)
 	dbpool.create (_logger, backend = backend)
 	adns.init (_logger)
 	lifetime.init (_timeout / 2., logger) # maintern interval
+	if tracking:
+		lifetime.enable_memory_track ()
 	_initialized = True
 
 def _reque_first (request):
@@ -143,10 +145,12 @@ def handle_status_401 (response):
 	global _que
 	if not response.request.get_auth () or response.request.reauth_count:
 		return response
-	response.request.reauth_count = 1
 	
-	_logger ("authorization failed, %s" % response.url, "info")
-	_reque_first (response.request)
+	_logger ("authorization failed, %s" % response.url, "info")		
+	request = response.request	
+	request.reauth_count = 1	
+	del response
+	_reque_first (request)	
 
 def handle_status_3xx (response):
 	global _allow_redirects	, _que
@@ -158,54 +162,62 @@ def handle_status_3xx (response):
 
 	newloc = response.get_header ('location')	
 	oldloc = response.request.uri
-	original_request = response.request
+	request = response.request
 	
 	if newloc == oldloc:
-		response.response = http_response.FailedResponse (711, "Redirect Error", original_request)
+		response.response = http_response.FailedResponse (711, "Redirect Error", request)
 		return response
 	
-	try:
-		new_request = response.request.relocate (response.response, newloc)
+	try:		
+		request.relocate (response.response, newloc)		
 	except RuntimeError:		
-		response.response = http_response.FailedResponse (711, "Redirect Error", original_request)
+		response.response = http_response.FailedResponse (711, "Redirect Error", request)
 		return response
 	
-	_logger ("%s redirected to %s from %s" % (response.status_code, newloc, oldloc), "info")	
+	#_logger ("%s redirected to %s from %s" % (response.status_code, newloc, oldloc), "info")	
 	# DO NOT use relocated response.request, it is None
-	_reque_first (new_request)
+	del response
+	_reque_first (request)
 	
 def _request_finished (handler):
 	global _cb_gateway, _currents, _concurrent, _finished_total, _logger, _bytesrecv,_force_h1
-	try:
-		if isinstance (handler, dbo_request.Request):
-			response = handler
-			_currents.pop (response.meta ['req_id'])
-			
-		else:
-			response = response_builder.HTTPResponse (handler)
-			_currents.pop (response.meta ['req_id'])
+	
+	if isinstance (handler, dbo_request.Request):
+		response = handler	
+		_currents.pop (response.meta ['req_id'])
+	else:
+		response = response_builder.HTTPResponse (handler)
+		_currents.pop (response.meta ['req_id'])
+		
+		try:
 			for handle_func in (handle_status_401, handle_status_3xx):
 				response = handle_func (response)
 				if not response:
-					return		
-		_finished_total += 1
-		response.logger = _logger
-		_bytesrecv += len (response.content)
-		callback = response.meta ['req_callback'] or _cb_gateway
-		try:
-			callback (response)
-		except:		
-			_logger.trace ()	
+					return
+		except:
+			_logger.trace ()
+		
+	_finished_total += 1
+	response.logger = _logger
+	_bytesrecv += len (response.content)
+	callback = response.meta ['req_callback'] or _cb_gateway
+	try:
+		callback (response)
+	except:		
+		_logger.trace ()	
 	
+	# clearing memory
+	handler.request.meta = None
+	del response
+	
+	try:
+		qsize () and _req ()		
 	except RecursionError:
 		try: 
 			_currents.pop (handler.request.meta ['req_id'])
 		except KeyError: 
 			pass	
-		_logger ("too many error occured, stop requeueing", "info")	
-		
-	else:	
-		qsize () and _req ()
+		_logger ("too many error occured, failed requeueing", "fail")
 		
 def _req ():
 	global _que, _logger, _currents, _request_total, _backend
@@ -288,6 +300,10 @@ def concurrent ():
 def fetchall ():
 	global _workers, _logger, _que, _timeout, _max_conns, _bytesrecv, _concurrent, _finished_total, _max_conns, _force_h1	
 	
+	if not qsize ():
+		_logger.log ('no item in queue.')
+		return
+	
 	if not _initialized:
 		configure ()
 	
@@ -296,7 +312,7 @@ def fetchall ():
 	lifetime._polling = 1
 	
 	# create initail workers	
-	_logger ("creating connection pool", "info")
+	#_logger ("creating connection pool", "info")
 	target_socks = min (_workers, qsize ())
 	for i in range (target_socks):
 		_req ()
@@ -306,7 +322,7 @@ def fetchall ():
 		while qsize ():
 			lifetime.lifetime_loop (os.name == "nt" and 1.0 or _timeout / 2.0, 1)
 			if sum ([1 for conn in asyncore.socket_map.values () if not isinstance (conn, async_dns) and conn.get_proto () in H2_PROTOCOLS and conn.connected and not conn.isactive ()]) == _workers:
-				_logger ('%d connection(s) created' % target_socks, 'info')
+				#_logger ('%d connection(s) created' % target_socks, 'info')
 				break
 			
 	# now starting
@@ -377,14 +393,8 @@ def add (request):
 	_que.add (request)
 			
 #----------------------------------------------------
-# REST CALL
+# HTTP CALL
 #----------------------------------------------------	
-def get (*args, **karg):
-	_add ('get', *args, **karg)
-
-def delete (*args, **karg):
-	_add ('delete', *args, **karg)
-
 def head (*args, **karg):
 	_add ('head', *args, **karg)
 
@@ -393,61 +403,68 @@ def trace (*args, **karg):
 
 def options (*args, **karg):
 	_add ('options', *args, **karg)
-				
+
+def upload (*args, **karg):
+	_add ('upload', *args, **karg)
+
+
+def get (*args, **karg):
+	_add ('get', *args, **karg)
+
+def delete (*args, **karg):
+	_add ('delete', *args, **karg)
+
 def post (*args, **karg):
 	_add ('post', *args, **karg)
 
 def patch (*args, **karg):
 	_add ('patch', *args, **karg)
 
-def patchform (*args, **karg):
-	_add ('patchform', *args, **karg)
-
-def patchxml (*args, **karg):
-	_add ('patchxml', *args, **karg)
-
-def patchjson (*args, **karg):
-	_add ('patchjson', *args, **karg)
-
-def patchnvp (*args, **karg):
-	_add ('patchnvp', *args, **karg)
-					
 def put (*args, **karg):
 	_add ('put', *args, **karg)
 
-def postform (*args, **karg):
-	_add ('postform', *args, **karg)
 
-def postxml (*args, **karg):
-	_add ('postxml', *args, **karg)
+def getjson (*args, **karg):
+	_add ('getjson', *args, **karg)
 
+def deletejson (*args, **karg):
+	_add ('deletejson', *args, **karg)
+
+def patchjson (*args, **karg):
+	_add ('patchjson', *args, **karg)
+	
 def postjson (*args, **karg):
 	_add ('postjson', *args, **karg)	
-
-def postnvp (*args, **karg):
-	_add ('postnvp', *args, **karg)	
-
-def putform (*args, **karg):
-	_add ('putform', *args, **karg)
-
-def putxml (*args, **karg):
-	_add ('putxml', *args, **karg)
-
+	
 def putjson (*args, **karg):
 	_add ('putjson', *args, **karg)	
 
-def putnvp (*args, **karg):
-	_add ('putnvp', *args, **karg)	
-	
-def upload (*args, **karg):
-	_add ('upload', *args, **karg)	
 
+def getxml (*args, **karg):
+	_add ('getxml', *args, **karg)
+
+def deletexml (*args, **karg):
+	_add ('deletexml', *args, **karg)
+
+def patchxml (*args, **karg):
+	_add ('patchxml', *args, **karg)
+	
+def postxml (*args, **karg):
+	_add ('postxml', *args, **karg)	
+	
+def putxml (*args, **karg):
+	_add ('putxml', *args, **karg)	
+	
+
+#----------------------------------------------------
+# Websocket
+#----------------------------------------------------	
 def ws (*args, **karg):
 	_add ('ws', *args, **karg)	
 
 def wss (*args, **karg):
 	_add ('wss', *args, **karg)	
-	
+
 	
 #----------------------------------------------------
 # XMLRPC, gRPC
@@ -478,7 +495,7 @@ def _adddbo (method, dbmethod, params, server, dbname = None, auth = None, callb
 	
 def postgresql (*args, **karg):
 	return stubproxy.Proxy ('postgresql', _adddbo, *args, **karg)
-pg = postgresql
+pgsql = pg = postgresql
 
 def redis (*args, **karg):
 	return stubproxy.Proxy ('redis', _adddbo, *args, **karg)
