@@ -26,8 +26,7 @@ class AsynConnect (asynchat.async_chat):
 	ac_in_buffer_size = 65535
 	ac_out_buffer_size = 65535
 	zombie_timeout = 10
-	keep_alive = 10
-	backend_keep_alive = 300
+	keep_alive = 10	
 	request_count = 0
 	active = 0
 	fifo_class = deque
@@ -69,9 +68,9 @@ class AsynConnect (asynchat.async_chat):
 		new_asyncon.keep_connect = self.keep_connect
 		return new_asyncon
 	
-	def set_backend (self, flag = True):
-		self.backend = flag
-		self.keep_alive = self.backend_keep_alive
+	def set_backend (self, backend_keep_alive = 10):
+		self.backend = True
+		self.keep_alive = backend_keep_alive
 		
 	def set_auth (self, auth):
 		self.auth = auth
@@ -310,8 +309,8 @@ class AsynConnect (asynchat.async_chat):
 			else:
 				raise
 	
-	def close_if_over_keep_live (self):
-		if time.time () - self.event_time > self.zombie_timeout:
+	def close_if_over_keep_live (self):		
+		if time.time () - self.event_time > max (self.keep_alive - 10, 5):
 			self.disconnect ()
 		
 	def set_timeout (self, timeout = 10):
@@ -338,7 +337,7 @@ class AsynConnect (asynchat.async_chat):
 		else:
 			self.handle_expt ()
 	
-	def maintern (self, object_timeout):
+	def maintern (self, object_timeout):		
 		if time.time () - self.event_time > object_timeout:
 			if self.handler and hasattr (self.handler, "enter_shutdown_process"):
 				self.handler.enter_shutdown ()
@@ -350,12 +349,18 @@ class AsynConnect (asynchat.async_chat):
 		
 	# proxy POST need no init_send
 	def push (self, thing, init_send = True):
+		if self.connected:
+			self.close_if_over_keep_live () # check keep-alive
+			
 		if isinstance(thing, (bytes, bytearray, memoryview)):
 			asynchat.async_chat.push (self, thing)
 		else:
 			self.push_with_producer (thing, init_send)	
 		
 	def push_with_producer (self, producer, init_send = True):
+		if self.connected:
+			self.close_if_over_keep_live () # check keep-alive
+			
 		self.producer_fifo.append (producer)
 		if init_send:
 			self.initiate_send ()
@@ -410,11 +415,8 @@ class AsynConnect (asynchat.async_chat):
 		self.handler = handler
 		if DEBUG:
 			self.logger ('begin_tran {rid:%s} %s' % (self.handler.request.meta.get ('req_id', -1), self.handler.request.uri), 'debug')
+		
 		self.set_event_time ()
-		
-		if self.connected:
-			self.close_if_over_keep_live () # check keep-alive
-		
 		# IMP: call add_channel () AFTER push()	otherwise threading issue will be raised
 		try:
 			if not self.connected:				
