@@ -12,7 +12,6 @@ class OperationalError (Exception):
 class DBConnect:
 	zombie_timeout = 120
 	keep_alive = 120
-	backend_keep_alive = 300
 	
 	def __init__ (self, address, params = None, lock = None, logger = None):
 		self.address = address
@@ -47,9 +46,13 @@ class DBConnect:
 		
 		self.set_event_time ()
 	
-	def set_backend (self, flag = True):
-		self.backend = flag
-		self.keep_alive = self.backend_keep_alive
+	def close_if_over_keep_live (self):
+		if time.time () - self.event_time > max (self.keep_alive - 10, 5):			
+			self.disconnect ()
+			
+	def set_backend (self, backend_keep_alive = 10):
+		self.backend = True
+		self.keep_alive = backend_keep_alive
 		
 	def duplicate (self):
 		new_asyncon = self.__class__ (self.address, self.params, self.lock, self.logger)
@@ -80,7 +83,7 @@ class DBConnect:
 		if self.isactive ():
 			return 1
 		else:
-			self.handle_close ()
+			self.close ()
 			self.__no_more_request = False
 			return 0
 	
@@ -213,7 +216,8 @@ class DBConnect:
 		self.exception_str = ""
 		self.exception_class = None		
 		self.execute_count += 1
-		self.set_event_time ()		
+		self.close_if_over_keep_live ()
+		self.set_event_time ()
 		
 	def execute (self, request):		
 		self.begin_tran (request)		
