@@ -9,7 +9,8 @@ import gzip
 from . import compressors, strutil
 import mimetypes
 import os
-from collections import Iterable
+from collections import Iterable, deque
+import asynchat
 
 """
 A collection of producers.
@@ -20,12 +21,18 @@ For example, you can feed dynamically-produced output into the compressing
 producer, then wrap this with the 'chunked' transfer-encoding producer.
 """
 
+SIZE_BUFFER = asynchat.async_chat.ac_out_buffer_size
+
 class simple_producer:
 	"producer for a string"
-	def __init__ (self, data, buffer_size = 4096):
+	def __init__ (self, data, buffer_size = SIZE_BUFFER):
 		self.data = data
 		self.buffer_size = buffer_size
-
+		self.proxsize = len (data)
+	
+	def get_size (self):
+		return self.proxsize
+		
 	def more (self):
 		if len (self.data) > self.buffer_size:
 			result = self.data[:self.buffer_size]
@@ -39,6 +46,7 @@ class simple_producer:
 class list_producer (simple_producer):
 	def __init__ (self, data):
 		self.data = data
+		self.proxsize = sum ([len (each) for each in self.data])
 		
 	def more (self):
 		if not self.data:
@@ -49,6 +57,10 @@ class list_producer (simple_producer):
 		return data
 
 class iter_producer (list_producer):
+	def __init__ (self, data):
+		self.data = data
+		self.proxsize = -1
+	
 	def more (self):
 		try:
 			data = next (self.data)
@@ -57,18 +69,18 @@ class iter_producer (list_producer):
 			return data
 		except StopIteration:		
 			return b""
-
+	
 class closing_stream_producer (simple_producer):
-	def __init__ (self, data, buffer_size = 4096):
-		if not hasattr (data, "read"):
-			raise AttributeError ("stream object should have `read()` returns bytes object and optional 'close()'")			
-		simple_producer.__init__ (self, data, buffer_size)
+	def __init__ (self, data, buffer_size = SIZE_BUFFER):
+		self.data = data
+		self.buffer_size = buffer_size
 		self.closed = False
+		self.proxsize = hasattr (data, "get_size") and data.get_size () or -1
 			
 	def more (self):
 		if self.closed: 
 			return b""
-		data = self.data.read (self.buffer_size)
+		data = self.data.read (self.buffer_size)		
 		if not data:
 			self.close ()
 		return data
@@ -81,11 +93,15 @@ class closing_stream_producer (simple_producer):
 	
 class scanning_producer:
 	"like simple_producer, but more efficient for large strings"
-	def __init__ (self, data, buffer_size = 4096):
+	def __init__ (self, data, buffer_size = SIZE_BUFFER):
 		self.data = data
 		self.buffer_size = buffer_size
+		self.proxsize = len (self.data)
 		self.pos = 0
 
+	def get_size (self):
+		return self.proxsize
+		
 	def more (self):
 		if self.pos < len(self.data):
 			lp = self.pos
@@ -104,7 +120,10 @@ class lines_producer:
 
 	def __init__ (self, lines):
 		self.lines = lines
-
+	
+	def get_size (self):
+		return sum ([len (each) for each in self.data])
+		
 	def ready (self):
 		return len(self.lines)
 
@@ -122,10 +141,12 @@ class buffer_list_producer:
 	# i.e., data == string.join (buffers, '')
 	
 	def __init__ (self, buffers):
-
 		self.index = 0
 		self.buffers = buffers
-
+		
+	def get_size (self):
+		return len (self.buffers)
+		
 	def more (self):
 		if self.index >= len(self.buffers):
 			return b''
@@ -138,12 +159,21 @@ class file_producer:
 	"producer wrapper for file[-like] objects"
 
 	# match http_channel's outgoing buffer size
-	buffer_size = 4096
-
-	def __init__ (self, file):
+	def __init__ (self, file, buffer_size = SIZE_BUFFER, proxsize = -1):
 		self.done = 0
 		self.file = file
-
+		self.buffer_size = buffer_size
+		self.proxsize = proxsize
+	
+	def get_size (self):
+		if self.proxsize != -1:
+			return self.proxsize
+		# possible io object
+		try:
+			return os.path.getsize (self.file.name)
+		except:
+			return -1	
+			
 	def more (self):
 		if self.done:
 			return b''
@@ -157,102 +187,6 @@ class file_producer:
 			else:
 				return data
 
-
-"""-----------------------------286502649418924
-Content-Disposition: form-data; name="submit-hidden"
-
-Genious
------------------------------286502649418924
-Content-Disposition: form-data; name="submit-name"
-
-Hans Roh
------------------------------286502649418924
-Content-Disposition: form-data; name="file1"; filename="House.2x01.acceptance.DVDRip-iMa.smi"
-Content-Type: application/smil
-
-asdadasda
------------------------------286502649418924
-Content-Disposition: form-data; name="file2"; filename=""
-Content-Type: application/octet-stream
-
-
------------------------------286502649418924--"""
-class multipart_producer:
-	buffer_size = 4096
-	
-	def __init__ (self, data, boundary):
-		# self.data = {"name": "Hans Roh", "file1": <open (path, "rb")>}
-		self.data = data
-		self.dlist = []
-		self.boundary = boundary		
-		self.current_file = None
-		self.content_length = self.calculate_size ()
-		#self.bytes_out = 0
-		
-	def calculate_size (self):
-		size = (len (self.boundary.encode ("utf8")) + 4) * (len (self.data) + 1) # --boundary + (\r\n or last --)
-		for name, value in list(self.data.items ()):
-			size += (41 + len (name.encode ("utf8"))) #Content-Disposition: form-data; name=""\r\n
-			if type (value) is not type (""):
-				fsize = os.path.getsize (value.name)
-				size += (fsize + 2) # file size + \r\n
-				fn = os.path.split (value.name) [-1]
-				size += len (fn.encode ("utf8")) + 13 # ; filename=""\r\n
-				mimetype = mimetypes.guess_type (fn) [0]
-				if not mimetype:
-					mimetype = "application/octet-stream"
-				size += (16 + len (mimetype)) # Content-Type: application/octet-stream\r\n				
-				self.dlist.append ((1, name, (value.name, fn, mimetype)))
-				value.close ()
-			
-			else:
-				size += len (value.encode ("utf8")) + 2 # value + \r\n
-				self.dlist.append ((0, name, value))
-
-			size += 2 # header end \r\n	
-			
-		return size	
-
-	def get_content_length (self):
-		return self.content_length
-		
-	def more (self):
-		if not self.dlist: 
-			return b''
-			
-		if self.current_file:
-			d = self.current_file.read (self.buffer_size)
-			if d:
-				return d
-			else:
-				self.current_file.close ()
-				self.current_file = None
-				self.dlist.pop (0)
-				d = "\r\n"
-				if not self.dlist:
-					d += "--%s--" % self.boundary
-				#self.bytes_out += len (d)
-				return d.encode ("utf8")
-			
-		if self.dlist:
-			first = self.dlist [0]
-			if first [0] == 0: # formvalue
-				d = '--%s\r\nContent-Disposition: form-data; name="%s"\r\n\r\n%s\r\n' % (self.boundary, first [1], first [2])
-				self.dlist.pop (0)
-				if not self.dlist:
-					d += "--%s--" % self.boundary
-				#self.bytes_out += len (d)
-				return d.encode ("utf8")
-			else:				
-				path, filename, mimetype = first [2]
-				self.current_file = open (path, "rb")
-				return ('--%s\r\nContent-Disposition: form-data; name="%s"; filename="%s"\r\nContent-Type: %s\r\n\r\n' % (
-					self.boundary, first [1], filename, mimetype
-				)).encode ("utf8")
-				#self.bytes_out += len (d)
-				#return d
-		
-
 # A simple output producer.  This one does not [yet] have
 # the safety feature builtin to the monitor channel:  runaway
 # output will not be caught.
@@ -262,22 +196,28 @@ class multipart_producer:
 
 class output_producer:
 	"Acts like an output file; suitable for capturing sys.stdout"
+	
 	def __init__ (self):
 		self.data = ''
-			
+		self.proxsize = 0
+	
+	def get_size (self):
+		return self.proxsize
+				
 	def write (self, data):
 		lines = data.split (b'\n')
 		data = b'\r\n'.join (lines)
+		self.proxsize += len (data)
 		self.data = self.data + data
 		
 	def writeline (self, line):
+		self.proxsize += (len (line) + 2)
 		self.data = self.data + line + b'\r\n'
 		
 	def writelines (self, lines):
-		self.data = self.data + string.joinfields (
-			lines,
-			b'\r\n'
-			) + b'\r\n'
+		d = b'\r\n'.join (lines)
+		self.proxsize += (len (d) + 2)
+		self.data = self.data + d + b'\r\n'
 
 	def ready (self):
 		return (len (self.data) > 0)
@@ -300,32 +240,40 @@ class output_producer:
 class composite_producer:
 	"combine a fifo of producers into one"
 	def __init__ (self, producers):
-		self.producers = producers
-		self.bind_ready ()
+		self.producers = producers		
+		self.override ()
 	
-	def bind_ready (self):
-		if len(self.producers) == 0:
+	def get_size (self):
+		return self.estimate_size ()
+		
+	def estimate_size (self):
+		size = 0
+		for p in self.producers:
+			s = p.get_size () 
+			if s == -1:
+				return -1
+			size += s
+		return size
+			
+	def override (self):
+		if len (self.producers) == 0:
 			return
-		p = self.producers.first()
+		p = self.producers.first ()
 		if hasattr (p, "ready"):
 			self.ready = p.ready
 			p.ready = None
-			
-		else:
-			try:
-				del self.ready
-			except AttributeError:
-				pass		
-				
+		elif hasattr (self, 'ready'):
+			del self.ready
+						
 	def more (self):
-		while len(self.producers):
+		while len (self.producers):
 			p = self.producers.first()
 			d = p.more()			
 			if d:
 				return d
 			else:
 				self.producers.pop()
-				self.bind_ready ()
+				self.override ()
 		else:
 			return b''
 
@@ -336,14 +284,15 @@ class globbing_producer:
 	gain about 30% performance on requests to a single channel]
 	"""
 
-	def __init__ (self, producer, buffer_size = 4096):
+	def __init__ (self, producer, buffer_size = SIZE_BUFFER):
 		self.producer = producer
+		if hasattr (self.producer, "ready"):
+			raise TypeError ("globbing producer cannot have ready methods")
 		self.buffer = b''
 		self.buffer_size = buffer_size
-		
-		if hasattr (producer, "ready"):
-			self.ready = producer.ready
-			producer.ready = None
+	
+	def get_size (self):
+		return self.producer.get_size ()
 		
 	def more (self):
 		while len(self.buffer) < self.buffer_size:
@@ -357,28 +306,28 @@ class globbing_producer:
 		return r
 
 
-class hooked_producer:
+class hooked_producer (globbing_producer):
 	"""
 	A producer that will call <function> when it empties,.
 	with an argument of the number of bytes produced.  Useful
 	for logging/instrumentation purposes.
 	"""
-
 	def __init__ (self, producer, function):
 		self.producer = producer
 		self.function = function
-		self.bytes = 0
-		
-		if hasattr (producer, "ready"):
-			self.ready = producer.ready
-			producer.ready = None
-
+		self.bytes = 0		
+		self.override ()
+	
+	def override (self):	
+		if hasattr (self.producer, "ready"):
+			self.ready = self.producer.ready
+			self.producer.ready = None
+				
 	def more (self):
-		#print "hooked_producer.more ()"
 		if self.producer:
-			result = self.producer.more()
-			if not result:
-				self.producer = None
+			result = self.producer.more()			
+			if not result:				
+				self.producer = None				
 				self.function (self.bytes)
 			else:
 				self.bytes = self.bytes + len(result)
@@ -394,7 +343,7 @@ class hooked_producer:
 # Blessing, and it really ought to be used even with normal files.
 # How beautifully it blends with the concept of the producer.
 
-class chunked_producer:
+class chunked_producer (hooked_producer):
 	"""A producer that implements the 'chunked' transfer coding for HTTP/1.1.
 	Here is a sample usage:
 		request['Transfer-Encoding'] = 'chunked'
@@ -407,15 +356,11 @@ class chunked_producer:
 	def __init__ (self, producer, footers=None):
 		self.producer = producer
 		self.footers = footers
-		
-		if hasattr (producer, "ready"):
-			self.ready = producer.ready
-			producer.ready = None
+		self.override ()
 			
 	def more (self):
 		if self.producer:
 			data = self.producer.more()
-			#print "------------", len (data)
 			if data:
 				dlen = '%x' % len (data)
 				return dlen.encode ("utf8") + b"\r\n" + data + b'\r\n'
@@ -440,7 +385,7 @@ try:
 except ImportError:
 	zlib = None
 
-class compressed_producer:
+class compressed_producer (hooked_producer):
 	"""
 	Compress another producer on-the-fly, using ZLIB
 	[Unfortunately, none of the current browsers seem to support this]
@@ -457,13 +402,8 @@ class compressed_producer:
 	def __init__ (self, producer, level=6):
 		self.producer = producer
 		self.compressor = zlib.compressobj (level, zlib.DEFLATED)
-		self.bind_ready ()
+		self.override ()
 	
-	def bind_ready (self):	
-		if hasattr (self.producer, "ready"):
-			self.ready = self.producer.ready
-			self.producer.ready = None
-			
 	def more (self):
 		if self.producer:
 			cdata = b''
@@ -484,8 +424,8 @@ class gzipped_producer (compressed_producer):
 	def __init__ (self, producer, level=5):
 		self.producer = producer
 		self.compressor = compressors.GZipCompressor (level)		
-		self.bind_ready ()
-			
+		self.override ()
+
 	
 class escaping_producer:
 
@@ -498,12 +438,12 @@ class escaping_producer:
 		self.esc_to = esc_to
 		self.buffer = b''
 		from asynchat import find_prefix_at_end
-		self.find_prefix_at_end = find_prefix_at_end
+		self.find_prefix_at_end = find_prefix_at_end		
+		self.override ()
+	
+	def get_size (self):
+		return self.producer.get_size ()
 		
-		if hasattr (producer, "ready"):
-			self.ready = producer.ready
-			producer.ready = None
-			
 	def more (self):
 		esc_from = self.esc_from
 		esc_to   = self.esc_to
@@ -524,34 +464,44 @@ class escaping_producer:
 		else:
 			return buffer
 
-
 class fifo:
-	def __init__ (self, list=None):
+	def __init__(self, list=None):
 		if not list:
-			self.list = []
+			self.list = deque()
 		else:
-			self.list = list
+			self.list = deque(list)
+	
+	def get_estimate_content_length (self):
+		cl = 0
+		for p in self.list:
+			s = p.get_size ()
+			if s == -1:
+				return -1				
+			cl += s	
+		return cl
 		
-	def __len__ (self):
+	def __len__(self):
 		return len(self.list)
 
-	def first (self):
+	def is_empty(self):
+		return not self.list
+
+	def first(self):
 		return self.list[0]
 
-	def push_front (self, object):
-		self.list.insert (0, object)
-
-	def push (self, data):
-		self.list.append (data)
-
-	def pop (self):
+	def push(self, data):
+		self.list.append(data)
+	
+	def push_front (self, data):
+		self.push(data)
+		self.list.rotate (1)
+		
+	def pop(self):
 		if self.list:
-			result = self.list[0]
-			del self.list[0]
-			return (1, result)
+			return (1, self.list.popleft())
 		else:
 			return (0, None)
 	
 	def clear (self):
-		self.list = []
-			
\ No newline at end of file
+		self.list.clear ()
+				
\ No newline at end of file
