@@ -5,6 +5,11 @@ import os
 import bisect
 import socket
 import time
+import types
+try:
+	from pympler import muppy, summary, tracker
+except ImportError:
+	pass	
 
 if os.name == "nt":
 	from errno import WSAENOTSOCK
@@ -45,7 +50,7 @@ class Maintern:
 			#bisect.insort (self.q, (now + interval, interval, func, args))
 			self.q.append ((now + interval, interval, func, args))
 			self.q.sort (key = lambda x: x [0])
-
+		
 def maintern_gc (now):
 	gc.collect ()
 	
@@ -75,6 +80,25 @@ def init (kill_zombie_interval = 10.0, logger = None):
 	maintern.sched (kill_zombie_interval, maintern_zombie_channel)
 	maintern.sched (300.0, maintern_gc)
 
+summary_tracker = None
+def summary_track (now):
+	global summary_tracker
+	
+	all_objects = muppy.get_objects ()
+	#all_objects = muppy.filter (all_objects, Type = dict)
+	#for each in all_objects:
+	#	print (each)
+	sum1 = summary.summarize (all_objects)
+	summary.print_ (sum1)
+	print ('-' * 79)	
+	if summary_tracker is None:
+		summary_tracker = tracker.SummaryTracker ()
+	summary_tracker.print_diff ()	
+	
+def enable_memory_track (interval = 10.0):
+	global maintern	
+	maintern.sched (interval, summary_track)	
+
 def shutdown (exit_code, shutdown_timeout = 30.0):
 	global _shutdown_phase
 	global _shutdown_timeout
@@ -114,10 +138,7 @@ def loop (timeout = 30.0):
 	
 	_polling = 0
 	
-if hasattr(select, 'poll'):
-	poll_fun = asyncore.poll2
-else:
-	poll_fun = asyncore.poll
+poll_fun = asyncore.poll
 
 def remove_notsocks (map):
 	global _select_errors, _logger
@@ -125,28 +146,34 @@ def remove_notsocks (map):
 	# on Windows we can get WSAENOTSOCK if the client
 	# rapidly connect and disconnects
 	killed = 0
-	for fd in list(map.keys()):
-		try:
-			select.select([fd], [], [], 0)
-			
+	for fd, obj in list(map.items()):	
+		r = []; w = []; e = []	
+		is_r = obj.readable()
+		is_w = obj.writable()
+		if is_r:
+			r = [fd]
+		# accepting sockets should not be writable
+		if is_w and not obj.accepting:
+			w = [fd]
+		if is_r or is_w:
+			e = [fd]
+		
+		try:			
+			select.select (r, w, e, 0)			
+									
 		except:
-			_logger and _logger.trace ()
+			#_logger and _logger.trace ()			
 			killed += 1
 			_select_errors += 1
 						
 			try:
-				obj = map [fd]
-			except KeyError:
-				pass
-			else:
-				try:
-					try: obj.handle_expt ()
-					except: obj.handle_error ()
-				except:
-					_logger and _logger.trace ()
-					
-				try: del map [fd]
-				except KeyError: pass
+				try: obj.handle_expt ()
+				except: obj.handle_error ()
+			except:
+				_logger and _logger.trace ()
+				
+			try: del map [fd]
+			except KeyError: pass
 					
 	return killed
 
@@ -155,10 +182,10 @@ def poll_fun_wrap (timeout, map = None):
 	
 	if map is None:
 		map = asyncore.socket_map
-		
-	try:		
-		poll_fun (timeout, map)
 	
+	try:		
+		poll_fun (timeout, map)		
+		
 	except (TypeError, OSError) as why:
 		# WSAENOTSOCK
 		remove_notsocks (map)
