@@ -36,7 +36,7 @@ class cachable_xmlrpc_buffer:
 			return self.cdata
 		res = self.buf.close ()
 		if self.cache:
-			self.cdata = res
+			self.cdata = res		
 		return res
 	
 	def raw (self):
@@ -51,40 +51,45 @@ class cachable_xmlrpc_buffer:
 class list_buffer:
 	def __init__(self, cache = 0):
 		self.cache = cache
-		self.raw = []
-		self.cdata = None
-	
+		self.data = []
+		self.cdata = b''
+		self.fp = None
+		
 	def __len__ (self):
 		return len (self.data)
 		
 	def feed(self, data):
-		self.raw.append(data)
+		if self.cache:
+			self.cdata += data
+		self.data.append (data)
 	
 	def raw (self):
-		f = BytesIO (b"".join (self.raw))
-		f.seek (0)
-		return f
+		if self.cdata:
+			self.fp = BytesIO (self.cdata)			
+		else:
+			self.fp = BytesIO (b"".join (self.data))
+		self.fp.seek (0)
+		return self.fp
 		
 	def read (self):	
 		# consume data, used by proxy response
-		data = self.build_data ()
-		self.raw = []
+		data = b''.join(self.data)
+		self.data = []
 		return data
 		
 	def build_data (self):
-		return b''.join(self.raw)
+		if self.cdata:
+			return self.cdata
+		return b''.join(self.data)
 	
 	def close (self):
 		if self.cdata:
 			return self.cdata
-		res = self.build_data ()
-		if self.cache:
-			self.cdata = res
-		return res
+		return self.build_data ()		
 	
 	def no_cache (self):
 		self.cache = 0
-		self.cdata = []
+		self.cdata = b''
 
 
 class bytes_buffer:
@@ -92,8 +97,8 @@ class bytes_buffer:
 		self.cache = cache
 		self.fp = BytesIO ()
 		self.current_buffer_size = 0
-		self.cdata = None		
-	
+		self.cdata = None
+		
 	def __len__ (self):
 		return self.current_buffer_size
 	
@@ -113,36 +118,12 @@ class bytes_buffer:
 			return self.cdata
 		res = self.build_data ()
 		if self.cache:
-			self.cdata = res
+			self.cdata = res		
 		return res
 	
 	def no_cache (self):
 		self.cache = 0
 		self.cdata = []
-
-
-class grpc_buffer (bytes_buffer):
-	def build_data (self):
-		msgs = []
-		fp = self.fp
-		fp.seek (0)
-		
-		byte = fp.read (1)
-		while byte:
-			iscompressed = struct.unpack ("<B", byte) [0]
-			length = struct.unpack ("<i", fp.read (4)) [0]
-			msg = fp.read (length)
-			msgs.append (msg)
-			byte = fp.read (1)
-		return tuple (msgs)
-	
-	def close (self):
-		if self.cdata:
-			return self.cdata
-		res = self.build_data ()
-		if self.cache:
-			self.cdata = res
-		return res
 		
 def getfakeparser (target_class, cache = False):
 	target = target_class (cache)
